package com.podpoint.pptmapview;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationManager;
import android.support.v4.content.ContextCompat;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.maps.android.clustering.ClusterManager;
import com.google.maps.android.clustering.Cluster;
import com.google.maps.android.clustering.ClusterManager.OnClusterClickListener;
import com.podpoint.pptmapview.PPTGoogleMarkerRenderer;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;
import java.util.HashMap;

public class PPTGoogleMapManager extends SimpleViewManager<PPTGoogleMapView> implements
        OnMapReadyCallback,
        GoogleMap.OnCameraChangeListener,
        GoogleMap.OnMapClickListener,
        GoogleMap.OnMapLongClickListener,
        GoogleMap.OnMarkerClickListener,
        GoogleMap.OnMarkerDragListener,
        GoogleMap.OnMyLocationButtonClickListener
{
    /**
     * The name of the react component.
     */
    public static final String REACT_CLASS = "PPTGoogleMap";

    /**
     * Whether or not this is the first time the map has become ready.
     */
    private boolean firstMapReady = true;

    /**
     * The context of this view.
     */
    private ReactContext reactContext;

    /**
     * The google map view.
     */
    private PPTGoogleMapView mapView;

    /**
     * The android location manager.
     */
    private LocationManager locationManager;

    /**
     * The markers which are to be added to the map.
     */
    private ReadableArray markers;

    /**
     * The the location that the map's camera should be moved to when it is next updated.
     */
    private CameraUpdate cameraUpdate;

    /**
     * Whether next camera update should be animated or not
     */
    private boolean animateCameraUpdate;

    /**
     * Whether or not the user's location marker is enabled.
     */
    private boolean showsUserLocation = false;

    /**
     * Whether scroll gestures are enabled (default) or disabled.
     */
    private boolean scrollGestures = true;

    /**
     * Whether zoom gestures are enabled (default) or disabled.
     */
    private boolean zoomGestures = true;

    /**
     * Whether tilt gestures are enabled (default) or disabled.
     */
    private boolean tiltGestures = true;

    /**
     * Whether rotate gestures are enabled (default) or disabled.
     */
    private boolean rotateGestures = true;

    /**
     * Whether the compass button is enabled or disabled.
     */
    private boolean compassButton = true;

    /**
     * Whether the my location button has been enabled.
     */
    private boolean myLocationButton = true;

    /**
     * Store bitmap descriptors to avoid rendering same icon multiple times
     */
    final HashMap<String, BitmapDescriptor> bitMapDescriptors = new HashMap<String, BitmapDescriptor>();

    /**
     * Cluster manager class
     */
    private ClusterManager<PPTGoogleMapMarker> mClusterManager;

    /**
     * Keep track of whether camera position has been updated or not
     */
    private boolean hasCameraChanged = false;

    /**
     * Returns the name of the react module.
     *
     * @return String
     */
    @Override
    public String getName() {
        return REACT_CLASS;
    }

    /**
     * Implementation of the react create view instance method - returns the map view to react.
     *
     * @param context
     * @return PPTGoogleMapView
     */
    @Override
    protected PPTGoogleMapView createViewInstance(ThemedReactContext context) {

        reactContext = context;

        mapView = new PPTGoogleMapView(context);
        mapView.onCreate(null);
        mapView.onResume();

        if (ContextCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            locationManager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
        }

        return mapView;
    }

    /**
     * Event handler for when map is ready to receive update parameters.
     *
     * @param googleMap
     */
    @Override
    public void onMapReady(final GoogleMap googleMap) {
        // Set up cluster manager if not set already
        if (mClusterManager == null) {
            mClusterManager = new ClusterManager<PPTGoogleMapMarker>(reactContext, googleMap);
            mClusterManager.setRenderer(new PPTGoogleMarkerRenderer(reactContext, googleMap, mClusterManager, bitMapDescriptors));
        }

        // Set all other flags
        UiSettings settings = googleMap.getUiSettings();
        settings.setScrollGesturesEnabled(this.scrollGestures);
        settings.setZoomGesturesEnabled(this.zoomGestures);
        settings.setTiltGesturesEnabled(this.tiltGestures);
        settings.setRotateGesturesEnabled(this.rotateGestures);
        settings.setCompassEnabled(this.compassButton);

        hasCameraChanged = false;

        // Update the camera position if it's set. Add padding to set marker in top of screen
        if (cameraUpdate != null) {
            int offset = (int)(mapView.getHeight() * (40.0f / 100.0f));
            googleMap.setPadding(0, 0, 0, mapView.getHeight() - offset);

            if (animateCameraUpdate) {
                googleMap.animateCamera(cameraUpdate, 800, null);
            } else {
                googleMap.moveCamera(cameraUpdate);
            }

            googleMap.setPadding(0, 0, 0, 0);
            hasCameraChanged = true;
        }

        // Set cluster click listener to ClusterManager
        mClusterManager.setOnClusterClickListener(new OnClusterClickListener<PPTGoogleMapMarker>() {
            @Override
            public boolean onClusterClick(final Cluster<PPTGoogleMapMarker> cluster) {
                googleMap.animateCamera(CameraUpdateFactory.newLatLngZoom(
                        cluster.getPosition(),
                        (float) Math.floor(googleMap.getCameraPosition().zoom + 3)
                        ), 800,
                        null);
                return true;
            }
        });

        // Set cluster item click listener to ClusterManager
        mClusterManager.setOnClusterItemClickListener(new ClusterManager.OnClusterItemClickListener<PPTGoogleMapMarker>() {
            @Override
            public boolean onClusterItemClick(PPTGoogleMapMarker pptGoogleMapMarker) {
                handleClusterItemEvent("didBeginDraggingMarker", pptGoogleMapMarker);
                return true;
            }
        });

        // Add the markers
        addMapMarkers(googleMap);
        googleMap.setOnMarkerClickListener(mClusterManager);
        googleMap.setOnCameraChangeListener(this);

        // Attach the event handlers
        if (firstMapReady) {
            googleMap.setOnMapClickListener(this);
            googleMap.setOnMapLongClickListener(this);
            googleMap.setOnMarkerDragListener(this);
            googleMap.setOnMyLocationButtonClickListener(this);
            firstMapReady = false;
        }

        mClusterManager.cluster();

        googleMap.setMyLocationEnabled(this.myLocationButton);
        googleMap.getUiSettings().setMyLocationButtonEnabled(this.myLocationButton);
    }

    /**
     * Parses the marker data received from react and adds the new markers to the map.
     *
     * @param googleMap
     */
    private void addMapMarkers(GoogleMap googleMap) {
        int count = this.markers.size();
        boolean wasReset = false;

        for (int i = 0; i < count; i++) {
            LatLng latLng;
            String publicId;

            ReadableMap marker = this.markers.getMap(i);

            if (marker.hasKey("latitude") && marker.hasKey("longitude") && marker.hasKey("publicId")) {
                latLng = new LatLng(marker.getDouble("latitude"), marker.getDouble("longitude"));
                publicId = marker.getString("publicId");
            } else {
                // We've got nothing to work with here - ignore this marker!
                continue;
            }

            // Ensure old markers are removed if we're adding new ones
            if ( !wasReset) {
                mClusterManager.clearItems();
                googleMap.clear();
                wasReset = true;
            }

            if (marker.hasKey("icon")) {
                ReadableMap iconMeta = marker.getMap("icon");
                markerWithCustomIcon(googleMap, latLng, iconMeta.getString("uri"), publicId);
            } else {
                markerWithDefaultIcon(googleMap, latLng, publicId);
            }
        }
    }

    /**
     * Loads a marker icon via URL and places it on the map at the required position.
     * Checks if marker is loaded from a URL or a resource identifier (RN debug/release mode)
     *
     * @param googleMap
     * @param latLng
     * @param uri
     */
    private void markerWithCustomIcon(final GoogleMap googleMap, final LatLng latLng, String uri, final String publicId) {
        try {
            // Check if given URI is a resource identifier
            int resourceIdentifier = reactContext.getResources().getIdentifier(uri, "drawable", reactContext.getPackageName());

            String markerPath;

            if (resourceIdentifier == 0) {
                markerPath = uri;
            } else {
                markerPath = Integer.toString(resourceIdentifier);
            }

            // Build Picasso target
            Target target = buildCustomTargetObject(googleMap, latLng, markerPath);

            if (resourceIdentifier == 0) {
                // Load icon from URL
                Picasso.with(reactContext)
                        .load(uri)
                        .into(target);

                mClusterManager.addItem(new PPTGoogleMapMarker(latLng, publicId, resourceIdentifier, uri));

            } else {
                // Load icon from resource identifier
                Picasso.with(reactContext)
                        .load(resourceIdentifier)
                        .into(target);

                mClusterManager.addItem(new PPTGoogleMapMarker(latLng, publicId, resourceIdentifier, uri));
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            markerWithDefaultIcon(googleMap,latLng, publicId);
        }
    }

    /**
     * Builds a new Target class for Picasso
     *
     * @param googleMap
     * @param latLng
     * @param publicId
     * @return Target
     */
    private Target buildCustomTargetObject(final GoogleMap googleMap, final LatLng latLng, final String publicId)
    {
        return new Target() {
            @Override
            public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                // Check if bitmap has been already rendered
                BitmapDescriptor descriptor = bitMapDescriptors.get(publicId);

                if (descriptor == null) {
                    descriptor = BitmapDescriptorFactory.fromBitmap(bitmap);
                    bitMapDescriptors.put(publicId, descriptor);
                }
            }

            @Override
            public void onBitmapFailed(Drawable errorDrawable) {
                System.out.println("Failed to load bitmap");
            }

            @Override
            public void onPrepareLoad(Drawable placeHolderDrawable) {
                System.out.println("Preparing to load bitmap");
            }
        };
    }

    /**
     * Places the default red marker on the map at the required position.
     *
     * @param googleMap
     * @param latLng
     */
    private void markerWithDefaultIcon(GoogleMap googleMap, LatLng latLng, String publicId)
    {
        MarkerOptions options = new MarkerOptions();
        options.position(latLng);
        googleMap.addMarker(options);
    }

    /**
     * Sets the user's location marker, if it has been enabled.
     *
     * @param map
     * @param cameraPosition
     */
    @ReactProp(name = "cameraPosition")
    public void setCameraPosition(PPTGoogleMapView map, ReadableMap cameraPosition) {
        float zoom = (float) cameraPosition.getDouble("zoom");

        animateCameraUpdate = true;

        if (cameraPosition.hasKey("animation") && ! cameraPosition.getBoolean("animation")) {
            animateCameraUpdate = false;
        }

        if (cameraPosition.hasKey("auto") && cameraPosition.getBoolean("auto") && ! cameraPosition.hasKey("latitude")) {

            // Set camera position to user GPS location
            Location location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);

            // Don't update camera position if location can't be found or it's already been updated
            if (location == null || hasCameraChanged) {
                map.getMapAsync(this);
                return;
            }

            cameraUpdate = CameraUpdateFactory.newLatLngZoom(
                    new LatLng(location.getLatitude(), location.getLongitude()),
                    zoom
            );
        } else {

            // Set camera position manually
            cameraUpdate = CameraUpdateFactory.newLatLngZoom(
                    new LatLng(
                            cameraPosition.getDouble("latitude"),
                            cameraPosition.getDouble("longitude")
                    ), zoom
            );
        }

        map.getMapAsync(this);
    }

    /**
     * Adds marker icons to the map.
     *
     * @param map
     * @param markers
     */
    @ReactProp(name = "markers")
    public void setMarkers(PPTGoogleMapView map, ReadableArray markers) {
        this.markers = markers;
        mClusterManager = null;
        map.getMapAsync(this);
    }

    /**
     * Sets the user's location marker, if it has been enabled.
     *
     * @param showsUserLocation
     */
    @ReactProp(name = "showsUserLocation")
    public void setShowsUserLocation(PPTGoogleMapView map, boolean showsUserLocation) {
        this.showsUserLocation = showsUserLocation;
    }

    /**
     * Controls whether scroll gestures are enabled (default) or disabled.
     *
     * @param map
     * @param scrollGestures
     */
    @ReactProp(name = "scrollGestures")
    public void setScrollGestures(PPTGoogleMapView map, boolean scrollGestures) {
        this.scrollGestures = scrollGestures;
    }

    /**
     * Controls whether zoom gestures are enabled (default) or disabled.
     *
     * @param map
     * @param zoomGestures
     */
    @ReactProp(name = "zoomGestures")
    public void setZoomGestures(PPTGoogleMapView map, boolean zoomGestures) {
        this.zoomGestures = zoomGestures;
    }

    /**
     * Controls whether tilt gestures are enabled (default) or disabled.
     *
     * @param map
     * @param tiltGestures
     */
    @ReactProp(name = "tiltGestures")
    public void setTiltGestures(PPTGoogleMapView map, boolean tiltGestures) {
        this.tiltGestures = tiltGestures;
    }

    /**
     * Controls whether rotate gestures are enabled (default) or disabled.
     *
     * @param map
     * @param rotateGestures
     */
    @ReactProp(name = "rotateGestures")
    public void setRotateGestures(PPTGoogleMapView map, boolean rotateGestures) {
        this.rotateGestures = rotateGestures;
    }

    /**
     * Controls whether gestures by users are completely consumed by the map view when gestures are enabled (default YES).
     *
     * @param map
     * @param consumesGesturesInView
     */
    @ReactProp(name = "consumesGesturesInView")
    public void setConsumesGesturesInView(PPTGoogleMapView map, boolean consumesGesturesInView) {
        // Do nothing - this is an iOS feature that we're only implementing so that the Android
        // map package doesn't break.
    }

    /**
     * Enables or disables the compass.
     *
     * @param map
     * @param compassButton
     */
    @ReactProp(name = "compassButton")
    public void setCompassButton(PPTGoogleMapView map, boolean compassButton) {
        this.compassButton = compassButton;
    }

    /**
     * Enables or disables the My Location button.
     *
     * @param map
     * @param myLocationButton
     */
    @ReactProp(name = "myLocationButton")
    public void setMyLocationButton(PPTGoogleMapView map, boolean myLocationButton) {
        this.myLocationButton = myLocationButton;
    }

    /**
     * Called repeatedly during any animations or gestures on the map (or once, if the camera is
     * explicitly set). This may not be called for all intermediate camera positions. It is always
     * called for the final position of an animation or gesture.
     *
     * @param cameraPosition
     */
    @Override
    public void onCameraChange(CameraPosition cameraPosition) {
        WritableMap event = Arguments.createMap();
        WritableMap data = Arguments.createMap();

        data.putDouble("latitude", cameraPosition.target.latitude);
        data.putDouble("longitude", cameraPosition.target.longitude);
        data.putDouble("zoom", cameraPosition.zoom);

        event.putString("event", "didChangeCameraPosition");
        event.putMap("data", data);

        if (mClusterManager != null) {
            mClusterManager.onCameraChange(cameraPosition);
        }

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );
    }

    /**
     * Called after a tap gesture at a particular coordinate, but only if a marker was not tapped.
     *
     * @param latLng
     */
    @Override
    public void onMapClick(LatLng latLng) {
        WritableMap event = Arguments.createMap();
        WritableMap data = Arguments.createMap();

        data.putDouble("latitude", latLng.latitude);
        data.putDouble("longitude", latLng.longitude);

        event.putString("event", "didTapAtCoordinate");
        event.putMap("data", data);

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );
    }

    /**
     * Called after a long-press gesture at a particular coordinate.
     *
     * @param latLng
     */
    @Override
    public void onMapLongClick(LatLng latLng) {
        WritableMap event = Arguments.createMap();
        WritableMap data = Arguments.createMap();

        data.putDouble("latitude", latLng.latitude);
        data.putDouble("longitude", latLng.longitude);

        event.putString("event", "didLongPressAtCoordinate");
        event.putMap("data", data);

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );
    }

    /**
     * Called after a marker has been tapped.
     *
     * @param marker
     * @return
     */
    @Override
    public boolean onMarkerClick(Marker marker) {
        handleMarkerEvent("didBeginDraggingMarker", marker);
        return false;
    }

    /**
     * Called when dragging has been initiated on a marker.
     *
     * @param marker
     */
    @Override
    public void onMarkerDragStart(Marker marker) {
        handleMarkerEvent("didBeginDraggingMarker", marker);
    }

    /**
     * Called while a marker is dragged.
     *
     * @param marker
     */
    @Override
    public void onMarkerDrag(Marker marker) {
        handleMarkerEvent("didDragMarker", marker);
    }

    /**
     * Called after dragging of a marker ended.
     *
     * @param marker
     */
    @Override
    public void onMarkerDragEnd(Marker marker) {
        handleMarkerEvent("didEndDraggingMarker", marker);
    }

    private void handleClusterItemEvent(String eventName, PPTGoogleMapMarker clusterItem)
    {
        WritableMap event = Arguments.createMap();
        WritableMap data = Arguments.createMap();

        data.putDouble("latitude", clusterItem.getPosition().latitude);
        data.putDouble("longitude", clusterItem.getPosition().longitude);
        data.putString("publicId", clusterItem.name);

        event.putString("event", "didTapMarker");
        event.putMap("data", data);

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );
    }

    /**
     * Handles marker events by emitting react events.
     *
     * @param eventName
     * @param marker
     */
    private void handleMarkerEvent(String eventName, Marker marker) {
        WritableMap event = Arguments.createMap();
        WritableMap data = Arguments.createMap();

        data.putDouble("latitude", marker.getPosition().latitude);
        data.putDouble("longitude", marker.getPosition().longitude);
        event.putString("event", "didTapMarker");
        event.putMap("data", data);

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );
    }

    /**
     * Called when the My Location button is tapped. Returns YES if the listener has consumed the
     * event (i.e., the default behavior should not occur), NO otherwise (i.e., the default behavior
     * should occur). The default behavior is for the camera to move such that it is centered on the
     * user location.
     *
     * @return
     */
    @Override
    public boolean onMyLocationButtonClick() {
        WritableMap event = Arguments.createMap();

        event.putString("event", "didTapMyLocationButtonForMapView");

        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(
                mapView.getId(),
                "topChange",
                event
        );

        return false;
    }
}

