package com.podpoint.pptmapview;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.OvalShape;
import android.support.v4.content.ContextCompat;
import android.view.ViewGroup;
import com.facebook.react.bridge.ReactContext;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.maps.android.clustering.Cluster;
import com.google.maps.android.clustering.ClusterManager;
import com.google.maps.android.clustering.view.DefaultClusterRenderer;
import com.google.maps.android.ui.IconGenerator;
import com.google.maps.android.ui.SquareTextView;
import java.net.URL;
import java.util.HashMap;

/**
 * Draws profile photos inside markers (using IconGenerator).
 * When there are multiple people in the cluster, draw multiple photos (using MultiDrawable).
 */
public class PPTGoogleMarkerRenderer extends DefaultClusterRenderer<PPTGoogleMapMarker> {

    /**
     * Android UI IconGenerator class
     */
    private final IconGenerator mIconGenerator;

    /**
     * Clusters density.
     */
    private final float mDensity;

    /**
     * Cluster circle background shape.
     */
    private ShapeDrawable mColoredCircleBackground;

    /**
     * React context class
     */
    private ReactContext reactContext;

    /**
     * HashMap with rendered BitMapDescriptors to avoid rendering the same thing multiple times
     */
    private HashMap<String, BitmapDescriptor> bitMapDescriptors;

    /**
     * PPTGoogleMarkerRenderer constructor
     *
     * @param reactContext
     * @param googleMap
     * @param mClusterManager
     * @param bitMapDescriptors
     */
    public PPTGoogleMarkerRenderer(ReactContext reactContext, GoogleMap googleMap, ClusterManager mClusterManager, HashMap bitMapDescriptors) {
        super(reactContext, googleMap, mClusterManager);
        this.reactContext = reactContext;
        this.bitMapDescriptors = bitMapDescriptors;
        this.mDensity = reactContext.getResources().getDisplayMetrics().density;
        this.mIconGenerator = new IconGenerator(reactContext);
        this.mIconGenerator.setContentView(this.makeSquareTextView(reactContext));
        this.mIconGenerator.setTextAppearance(R.style.ClusterIcon_textStyle);
        this.mIconGenerator.setBackground(this.makeClusterBackground());
    }

    /**
     * Renders cluster items (markers)
     *
     * @param marker
     * @param markerOptions
     */
    @Override
    protected void onBeforeClusterItemRendered(PPTGoogleMapMarker marker, MarkerOptions markerOptions) {
        BitmapDescriptor descriptor = null;
        
        if (marker.image == 0) {
            descriptor =  this.bitMapDescriptors.get(marker.imageUrl);
        } else {
            descriptor =  this.bitMapDescriptors.get(marker.image);
        }

        if (descriptor == null) {
            Bitmap bm;

            if (marker.image == 0) {
                try {
                    URL uri = new URL(marker.imageUrl);
                    bm = BitmapFactory.decodeStream(uri.openConnection().getInputStream());
                    descriptor = BitmapDescriptorFactory.fromBitmap(bm);
                    this.bitMapDescriptors.put(marker.imageUrl, descriptor);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                bm = BitmapFactory.decodeResource(this.reactContext.getResources(), marker.image);
                descriptor = BitmapDescriptorFactory.fromBitmap(bm);
                this.bitMapDescriptors.put(Integer.toString(marker.image), descriptor);
            }
        }

        markerOptions.position(marker.getPosition())
                .icon(descriptor);
    }

    /**
     * Renders clusters (grouped markers).
     *
     * @param cluster
     * @param markerOptions
     */
    @Override
    protected void onBeforeClusterRendered(Cluster<PPTGoogleMapMarker> cluster, MarkerOptions markerOptions) {
        // Main color
        int clusterColor = ContextCompat.getColor(reactContext, R.color.white);

        this.mColoredCircleBackground.getPaint().setColor(clusterColor);
        BitmapDescriptor descriptor = BitmapDescriptorFactory.fromBitmap(
                this.mIconGenerator.makeIcon(String.valueOf(cluster.getSize())));

        markerOptions.icon(descriptor);
    }

    /**
     * Render cluster content view
     *
     * @param context
     * @return
     */
    private SquareTextView makeSquareTextView(Context context) {
        SquareTextView squareTextView = new SquareTextView(context);
        ViewGroup.LayoutParams layoutParams = new ViewGroup.LayoutParams(-2, -2);
        squareTextView.setLayoutParams(layoutParams);
        squareTextView.setId(R.id.text);
        int padding = (int)(8.0F * this.mDensity);
        squareTextView.setPadding(padding, padding, padding, padding);
        return squareTextView;
    }

    /**
     * Render cluster background view
     *
     * @return
     */
    private LayerDrawable makeClusterBackground() {
        // Outline color
        int clusterOutlineColor = ContextCompat.getColor(reactContext, R.color.colorPrimary);
        this.mColoredCircleBackground = new ShapeDrawable(new OvalShape());
        ShapeDrawable outline = new ShapeDrawable(new OvalShape());
        outline.getPaint().setColor(clusterOutlineColor);
        LayerDrawable background = new LayerDrawable(
            new Drawable[]{outline, this.mColoredCircleBackground}
        );
        int strokeWidth = (int)(this.mDensity * 4.0F);
        background.setLayerInset(1, strokeWidth, strokeWidth, strokeWidth, strokeWidth);
        return background;
    }

    /**
     * Work out whether current cluster should be rendered as cluster or multiple cluster items
     *
     * @param cluster
     * @return
     */
    @Override
    protected boolean shouldRenderAsCluster(Cluster cluster) {
        // Only render clusters that are close enough.
        return cluster.getSize() > 3;
    }
}