#import "PPTGoogleMap.h"

#import <React/RCTConvert.h>
#import <React/RCTEventDispatcher.h>
#import <React/RCTLog.h>
#import <React/RCTUtils.h>
#import "GDefaultClusterRenderer.h"
#import "NonHierarchicalDistanceBasedAlgorithm.h"
#import "Spot.h"

@implementation PPTGoogleMap {
    NSMutableDictionary *markerImages;
    CLLocationManager *locationManager;
    float zoom;
    GClusterManager *clusterManager_;
    PPTGoogleMapManager *mapManager_;
}

const int IPHONE_6_HEIGHT = 667;

/**
 * Init the google map view class.
 *
 * @return id
 */
- (id)init:(PPTGoogleMapManager *) mapManager
{
    if (self = [super init]) {
        markerImages = [[NSMutableDictionary alloc] init];
    }

    // Set up cluster manager
    clusterManager_ = [GClusterManager managerWithMapView:self
         algorithm:[[NonHierarchicalDistanceBasedAlgorithm alloc] init]
         renderer:[[GDefaultClusterRenderer alloc] initWithMapView:self]];

    self.delegate = clusterManager_;

    clusterManager_.delegate = mapManager;

    return self;
}

/**
 * Enables layout sub-views which are required to render a non-blank map.
 *
 * @return void
 */
- (void)layoutSubviews
{
    [super layoutSubviews];

    CGRect mapFrame = self.frame;

    self.frame = CGRectZero;
    self.frame = mapFrame;
}

#pragma mark Accessors

/**
 * Sets the map camera position.
 *
 * @return void
 */
- (void)setCameraPosition:(NSDictionary *)cameraPosition
{
    zoom = ((NSNumber*)cameraPosition[@"zoom"]).doubleValue;

    if (cameraPosition[@"latitude"] && cameraPosition[@"longitude"]) {

        // Reset map rotation
        [self animateToBearing:0];

        CLLocationDegrees latitude = ((NSNumber*)cameraPosition[@"latitude"]).doubleValue;
        CLLocationDegrees longitude = ((NSNumber*)cameraPosition[@"longitude"]).doubleValue;
        CLLocationCoordinate2D pos = CLLocationCoordinate2DMake(latitude, longitude);

        float height = [[UIScreen mainScreen] bounds].size.height;
        float heightProportion = height / IPHONE_6_HEIGHT;

        if (height > 667) {
            heightProportion += 0.15;
        } else if (height < 667) {
            heightProportion -= 0.15;
        } else {
            heightProportion += 0.05;
        }

        Float32 diff = 742 / (pow(2, zoom - 1));

        pos.latitude -= (0.1 * (diff * heightProportion));

        GMSCameraPosition *position = [GMSCameraPosition cameraWithTarget:pos zoom:zoom];

        Float32 animationDuration = 0.8;

        BOOL animation = [[cameraPosition objectForKey:@"animation"] boolValue];

        if (animation) {
            [CATransaction begin];
            [CATransaction setValue:[NSNumber numberWithFloat: (animationDuration / 3)] forKey:kCATransactionAnimationDuration];
            [self animateToLocation:pos];
            [CATransaction commit];

            [CATransaction begin];
            [CATransaction setValue:[NSNumber numberWithFloat: animationDuration] forKey:kCATransactionAnimationDuration];
            [self animateToZoom:zoom];
            [CATransaction commit];
        } else {
            GMSCameraPosition *position = [GMSCameraPosition cameraWithLatitude:pos.latitude longitude:pos.longitude zoom:zoom];
            [self setCamera:position];
        }

        self.didCameraUpdate = true;
    } else {

        if ( ! self.didCameraUpdate) {
            locationManager = [[CLLocationManager alloc] init];

            locationManager.delegate = self;
            locationManager.distanceFilter = kCLDistanceFilterNone;
            locationManager.desiredAccuracy = kCLLocationAccuracyBest;

            [locationManager startUpdatingLocation];
        }
    }
}

/**
 * The delegate for the did update location event - fired when the user's location becomes available and then centers the
 * map about the their location. The location manager is then stopped so that the map position doesn't continue to be updated.
 *
 * @return void
 */
- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
    GMSCameraPosition *camera = [GMSCameraPosition cameraWithLatitude:newLocation.coordinate.latitude
                                                            longitude:newLocation.coordinate.longitude
                                                                 zoom:zoom];

    if ( ! self.didCameraUpdate) {
        [self setCamera:camera];
    }

    [locationManager stopUpdatingLocation];
}

/**
 * Adds marker icons to the map.
 *
 * @return void
 */
- (void)setMarkers:(NSArray *)markers
{
    [clusterManager_ removeItems];

    for (NSDictionary* marker in markers) {
        NSString *publicId = marker[@"publicId"];
        CLLocationDegrees latitude = ((NSNumber*)marker[@"latitude"]).doubleValue;
        CLLocationDegrees longitude = ((NSNumber*)marker[@"longitude"]).doubleValue;

        GMSMarker* mapMarker = [GMSMarker markerWithPosition:CLLocationCoordinate2DMake(latitude, longitude)];

        mapMarker.icon = [self getMarkerImage:marker];
        mapMarker.userData = publicId;

        Spot* spot = [[Spot alloc] init];
        spot.location = CLLocationCoordinate2DMake(latitude, longitude);
        spot.marker = mapMarker;

        [clusterManager_ addItem:spot];
    }

    [clusterManager_ cluster];
}

/**
 * Load the marker image or use one that's already been loaded.
 *
 * @return NSImage
 */
- (UIImage *)getMarkerImage:(NSDictionary *)marker
{
    NSString *markerPath = marker[@"icon"][@"uri"];
    CGFloat markerScale = ((NSNumber*)marker[@"icon"][@"scale"]).doubleValue;

    if (!markerImages[markerPath]) {
        UIImage *markerImage;

        NSURL *markerUrl = [NSURL URLWithString:markerPath];

        NSURLRequest *request = [NSURLRequest requestWithURL:markerUrl];

        if([NSURLConnection canHandleRequest:request]) {
            markerImage = [UIImage imageWithData:[NSData dataWithContentsOfURL:markerUrl]];
        } else {
            markerImage = [UIImage imageWithData:[NSData dataWithContentsOfFile:markerUrl]];
        }

        UIImage *markerScaled = [UIImage imageWithCGImage:[markerImage CGImage]
                                                    scale:(markerScale)
                                              orientation:(markerImage.imageOrientation)];

        [markerImages setObject:markerScaled forKey:markerPath];
    }


    return markerImages[markerPath];
}

/**
 * Sets the user's location marker, if it has been enabled. Don't be alarmed if the marker looks funny when testing the app in
 * the simulator, there's a known bug: https://code.google.com/p/gmaps-api-issues/issues/detail?id=5472
 *
 * @return void
 */
- (void)setShowsUserLocation:(BOOL *)showsUserLocation
{
    if (showsUserLocation) {
        self.myLocationEnabled = YES;
    } else {
        self.myLocationEnabled = NO;
    }
}

/**
 * Controls whether scroll gestures are enabled (default) or disabled.
 *
 * @return void
 */
- (void)setScrollGestures:(BOOL *)scrollGestures
{
    if (scrollGestures) {
        self.settings.scrollGestures = YES;
    } else {
        self.settings.scrollGestures = NO;
    }
}

/**
 * Controls whether zoom gestures are enabled (default) or disabled.
 *
 * @return void
 */
- (void)setZoomGestures:(BOOL *)zoomGestures
{
    if (zoomGestures) {
        self.settings.zoomGestures = YES;
    } else {
        self.settings.zoomGestures = NO;
    }
}

/**
 * Controls whether tilt gestures are enabled (default) or disabled.
 *
 * @return void
 */
- (void)setTiltGestures:(BOOL *)tiltGestures
{
    if (tiltGestures) {
        self.settings.tiltGestures = YES;
    } else {
        self.settings.tiltGestures = NO;
    }
}

/**
 * Controls whether rotate gestures are enabled (default) or disabled.
 *
 * @return void
 */
- (void)setRotateGestures:(BOOL *)rotateGestures
{
    if (rotateGestures) {
        self.settings.rotateGestures = YES;
    } else {
        self.settings.rotateGestures = NO;
    }
}

/**
 * Controls whether gestures by users are completely consumed by the GMSMapView when gestures are enabled (default YES).
 *
 * @return void
 */
- (void)setConsumesGesturesInView:(BOOL *)consumesGesturesInView
{
    if (consumesGesturesInView) {
        self.settings.consumesGesturesInView = YES;
    } else {
        self.settings.consumesGesturesInView = NO;
    }
}

/**
 * Enables or disables the compass.
 *
 * @return void
 */
- (void)setCompassButton:(BOOL *)compassButton
{
    if (compassButton) {
        self.settings.compassButton = YES;
    } else {
        self.settings.compassButton = NO;
    }
}

/**
 * Enables or disables the My Location button.
 *
 * @return void
 */
- (void)setMyLocationButton:(BOOL *)myLocationButton
{
    if (myLocationButton) {
        self.settings.myLocationButton = YES;
    } else {
        self.settings.myLocationButton = NO;
    }
}

/**
 * Enables (default) or disables the indoor floor picker.
 *
 * @return void
 */
- (void)setIndoorPicker:(BOOL *)indoorPicker
{
    if (indoorPicker) {
        self.settings.indoorPicker = YES;
    } else {
        self.settings.indoorPicker = NO;
    }
}

/**
 * Controls whether rotate and zoom gestures can be performed off-center and scrolled around (default YES).
 *
 * @return void
 */
- (void)setAllowScrollGesturesDuringRotateOrZoom:(BOOL *)allowScrollGesturesDuringRotateOrZoom
{
    if (allowScrollGesturesDuringRotateOrZoom) {
        self.settings.allowScrollGesturesDuringRotateOrZoom = YES;
    } else {
        self.settings.allowScrollGesturesDuringRotateOrZoom = NO;
    }
}

- (GClusterManager *)getClusterManager
{
    return clusterManager_;
}

@end
